/*
 * This file is part of openKropki.
 * Copyright (C) 2012, 2013, 2014, 2015, Mateusz Viste
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef KROPKI_CORE_H_SENTINEL
  #define KROPKI_CORE_H_SENTINEL

  #define kropki_EMPTY 0     /* These values */
  #define kropki_PL1_DOT 1   /* must all fit */
  #define kropki_PL2_DOT 2   /* in 2 bits!   */

  /* a linked list node used to store walls within a kropki_game struct */
  struct kropki_wallelement {
    int x1, y1, x2, y2;
    int player;
    struct kropki_wallelement *next;
  };

  /* dots are stored as 1 byte per dot in the play field, where:
      bits ......XX = Indicates whether it's a dot for player 1, player 2 or an empty space.
      bits ....XX.. = Indicates whether the field is in captivity (captured by who)
      bits ..XX.... = Indicates whether the dot is part of a wall (wall's owner)
      bits XX...... = used for internal processing (mostly for applying the flooding algorithm) */

  struct kropki_game {
    struct kropki_wallelement *walls;
    int lastplayer; /* who played last (1 or 2) - 0 means nobody played yet */
    int lastmovex;  /* lastmove X coordinate */
    int lastmovey;  /* lastmove Y coordinate */
    int field_width;
    int field_height;
    unsigned char field[1]; /* This element MUST be the last in the struct. The struct */
  };                        /* will be allocated at runtime to accomodate field's size */
                            /* YES, this is safe - see ISO 9899-1990, 6.5.2.1.         */

  struct kropki_game *kropki_newgame(int width, int height);
  void kropki_freegame(struct kropki_game *game);
  int kropki_getpos(struct kropki_game *game, int x, int y);
  int kropki_setpos(struct kropki_game *game, int x, int y, int player);
  int kropki_setposforce(struct kropki_game *game, int x, int y, int player);
  void kropki_getpoints(struct kropki_game *game, int *scores);
  struct kropki_game *kropki_copygame(struct kropki_game *game);
  int kropki_findfirstfreefield(struct kropki_game *game, int *resx, int *resy);
#endif
